//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// This implements modules/weircd_api_services.h functions and helper functions
// that are needed by the implementations.
// Additionally, it provides functions to call the startup functions of a module,
// the config options check of a services module etc. (see modulesservicesapi.h)

#include "weircd.h"

/* Some function pointers we need */
typedef void (*defaultfunc)(void);
typedef int (*cprecheckfunc)(const char*, const char*);
typedef int (*cendcheckfunc)(void);
typedef int (*servicesstartupfunc)(int,const char**,const char**);

void servicesmodule_InitConfigPreCheck(const char* modname) {
	if (modman_GetPtrByName(modname) == NULL) {
		return;
	}
	module_Switched(modname);module_SetArea(MODAREA_GENERICFUNC);
	defaultfunc configcheck = modman_GetData(modname,"module_ServicesCOptionPreCheck_Begin");
	configcheck();
}
int servicesmodule_ConfigPreCheckOption(const char* modname, const char* option, const char* value) {
	module_Switched(modname);module_SetArea(MODAREA_GENERICFUNC);
	cprecheckfunc configcheck = modman_GetData(modname,"module_ServicesCOptionPreCheck");
	if (configcheck(option,value) == 0) {
		return 0;
	}
	return -1;
}
int servicesmodule_FinishConfigPreCheck(const char* modname) {
	module_Switched(modname);module_SetArea(MODAREA_GENERICFUNC);
	cendcheckfunc configcheck = modman_GetData(modname,"module_ServicesCOptionPreCheck_End");
	return configcheck();
}
int servicesmodule_CallStartup(const char* modname, int optioncount, const char** optionnames, const char** optionvalues) {
	module_Switched(modname);module_SetArea(MODAREA_GENERICFUNC);
	servicesstartupfunc func = modman_GetData(modname,"module_ServicesStartup");
	int i = func(optioncount,optionnames,optionvalues);
	return i;
}
void servicesmodule_CallShutdown(const char* modname) {
	module_Switched(modname);module_SetArea(MODAREA_GENERICFUNC);
	defaultfunc func = modman_GetData(modname,"module_ServicesShutdown");
	func();
}
int weircd_services_LoginFakeuserAsServiceBot(int handle, const char* nick, const char* userid, const char* host, const char* realname) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	void* ptr = modman_GetPtrByName(module_Current());
	if (ptr == NULL) {return -1;}
	void* modentry = FakeUsers_GetModuleFakeEntryByPtr(ptr);
	if (modentry == NULL) {return -1;}
	USER u = FakeUsers_GetUserPtr(handle, modentry);
	if (u == NOUSER) {return -1;}
	if (udat(u)->authed == 1 || udat(u)->pleaseremove == 1) {return -1;}
	if (irc_IsNicknameValid(nick) != 0) {return -1;}
	if (strlen(nick) > COMPILEOPTION_MAXNICKLEN) {return -1;}
	if (irc_IsNickUsed(nick) == 0) {return -1;}
	int r = 0;
	while (r < strlen(host)) {
		if (host[r] < 32 || host[r] == '!' || host[r] == '@' || host[r] > 126) {
			return -1;
		}
		r++;
	}
	if (strlen(host) < 1) {return -1;}
	char muserid[10] = "";
	strncpy(muserid,userid,9);muserid[9] = 0;
	cutonfirstspace(muserid);
	irc_MakeUserIdValid(muserid);
	if (strlen(muserid) <= 0) {return -1;}
	udat(u)->authed = 1;
	strncpy(udat(u)->nickname,nick,COMPILEOPTION_MAXNICKLEN - 1);
	udat(u)->nickname[COMPILEOPTION_MAXNICKLEN] = 0;
	strcpy(udat(u)->suserid,muserid);
	strncpy(udat(u)->realname,realname,COMPILEOPTION_MAXREALNAMELEN - 1);
	udat(u)->realname[COMPILEOPTION_MAXREALNAMELEN] = 0;
	
	/* now we are going to send stuff to ourselves. this will destroy
	   our module call restriction context, so we have to remember it */
	
	int a = module_GetArea();
	const char* p = module_Current();
	char* cpy = alloca(strlen(p)+1);
	strcpy(cpy,p);
	irc_Send001(u); //THIS!! will destroy all our
	module_Switched(cpy);
	module_SetArea(a);
	
	strncpy(udat(u)->host,host,COMPILEOPTION_MAXHOSTLEN-1);
	udat(u)->host[COMPILEOPTION_MAXHOSTLEN-1] = 0;
	udat(u)->mode_servicesbot = 1;
	udat(u)->mode_bot = 1;
	return 0;
}
void weircd_services_SetUserLoginAccountName(const char* user, const char* account) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return;}
	int previouslyregged = 0;
	if (udat(u)->accountname != NULL) {
		previouslyregged = 1;
		free(udat(u)->accountname);
	}
	if (account == NULL) {
		udat(u)->accountname = NULL;
		if (udat(u)->mode_fixedhost == 0) {
			host_Apply(u,host_GetDefault(u));
		}
		if (previouslyregged == 1) {
			usnd(u,":%s MODE %s -r",svname(),udat(u)->nickname);
		}
		return;
	}
	udat(u)->accountname = malloc(strlen(account)+1);
	if (udat(u)->accountname == NULL) {return;}
	strcpy(udat(u)->accountname,account);
	if (previouslyregged == 0) {
		usnd(u,":%s MODE %s +r",svname(),udat(u)->nickname);
	}
	if (udat(u)->mode_fixedhost == 0) {
		host_Apply(u,host_GetRegistered(u,account));
	}
}
int weircd_services_NickservKill(const char* ip) {
	return ips_NickservKill(ip);
}
void weircd_services_NickservLogin(const char* ip) {
	ips_NickservLogin(ip);
}